@@ -1,7 +1,7 @@
 /* Read ELF (Executable and Linking Format) object files for GDB.
 
    Copyright 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-   2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+   2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
    Written by Fred Fish at Cygnus Support.
 
@@ -83,12 +83,12 @@ elf_locate_sections (bfd *ignore_abfd, asection *sectp, void *eip)
   if (strcmp (sectp->name, ".debug") == 0)
     {
       ei->dboffset = sectp->filepos;
-      ei->dbsize = bfd_get_section_size_before_reloc (sectp);
+      ei->dbsize = bfd_get_section_size (sectp);
     }
   else if (strcmp (sectp->name, ".line") == 0)
     {
       ei->lnoffset = sectp->filepos;
-      ei->lnsize = bfd_get_section_size_before_reloc (sectp);
+      ei->lnsize = bfd_get_section_size (sectp);
     }
   else if (strcmp (sectp->name, ".stab") == 0)
     {
@@ -124,13 +124,14 @@ record_minimal_symbol (char *name, CORE_ADDR address,
 
    SYNOPSIS
 
-   void elf_symtab_read (struct objfile *objfile, int dynamic)
+   void elf_symtab_read (struct objfile *objfile, int dynamic,
+			 long number_of_symbols, asymbol **symbol_table)
 
    DESCRIPTION
 
-   Given an objfile and a flag that specifies whether or not the objfile
-   is for an executable or not (may be shared library for example), add
-   all the global function and data symbols to the minimal symbol table.
+   Given an objfile, a symbol table, and a flag indicating whether the
+   symbol table contains dynamic symbols, add all the global function
+   and data symbols to the minimal symbol table.
 
    In stabs-in-ELF, as implemented by Sun, there are some local symbols
    defined in the ELF symbol table, which can be used to locate
@@ -141,14 +142,12 @@ record_minimal_symbol (char *name, CORE_ADDR address,
  */
 
 static void
-elf_symtab_read (struct objfile *objfile, int dynamic)
+elf_symtab_read (struct objfile *objfile, int dynamic,
+		 long number_of_symbols, asymbol **symbol_table)
 {
   long storage_needed;
   asymbol *sym;
-  asymbol **symbol_table;
-  long number_of_symbols;
   long i;
-  struct cleanup *back_to;
   CORE_ADDR symaddr;
   CORE_ADDR offset;
   enum minimal_symbol_type ms_type;
@@ -162,293 +161,265 @@ elf_symtab_read (struct objfile *objfile, int dynamic)
   /* Name of filesym, as saved on the objfile_obstack.  */
   char *filesymname = obsavestring ("", 0, &objfile->objfile_obstack);
 #endif
-  struct dbx_symfile_info *dbx = objfile->sym_stab_info;
+  struct dbx_symfile_info *dbx = objfile->deprecated_sym_stab_info;
   int stripped = (bfd_get_symcount (objfile->obfd) == 0);
 
-  if (dynamic)
+  for (i = 0; i < number_of_symbols; i++)
     {
-      storage_needed = bfd_get_dynamic_symtab_upper_bound (objfile->obfd);
-
-      /* Nothing to be done if there is no dynamic symtab.  */
-      if (storage_needed < 0)
-	return;
-    }
-  else
-    {
-      storage_needed = bfd_get_symtab_upper_bound (objfile->obfd);
-      if (storage_needed < 0)
-	error ("Can't read symbols from %s: %s", bfd_get_filename (objfile->obfd),
-	       bfd_errmsg (bfd_get_error ()));
-    }
-  if (storage_needed > 0)
-    {
-      symbol_table = (asymbol **) xmalloc (storage_needed);
-      back_to = make_cleanup (xfree, symbol_table);
-      if (dynamic)
-	number_of_symbols = bfd_canonicalize_dynamic_symtab (objfile->obfd,
-							     symbol_table);
-      else
-	number_of_symbols = bfd_canonicalize_symtab (objfile->obfd, symbol_table);
-      if (number_of_symbols < 0)
-	error ("Can't read symbols from %s: %s", bfd_get_filename (objfile->obfd),
-	       bfd_errmsg (bfd_get_error ()));
-
-      for (i = 0; i < number_of_symbols; i++)
+      sym = symbol_table[i];
+      if (sym->name == NULL || *sym->name == '\0')
 	{
-	  sym = symbol_table[i];
-	  if (sym->name == NULL || *sym->name == '\0')
-	    {
-	      /* Skip names that don't exist (shouldn't happen), or names
-	         that are null strings (may happen). */
-	      continue;
-	    }
+	  /* Skip names that don't exist (shouldn't happen), or names
+	     that are null strings (may happen). */
+	  continue;
+	}
 
-          offset = ANOFFSET (objfile->section_offsets, sym->section->index);
-	  if (dynamic
-	      && sym->section == &bfd_und_section
-	      && (sym->flags & BSF_FUNCTION))
-	    {
-	      struct minimal_symbol *msym;
+      offset = ANOFFSET (objfile->section_offsets, sym->section->index);
+      if (dynamic
+	  && sym->section == &bfd_und_section
+	  && (sym->flags & BSF_FUNCTION))
+	{
+	  struct minimal_symbol *msym;
 
-	      /* Symbol is a reference to a function defined in
-	         a shared library.
-	         If its value is non zero then it is usually the address
-	         of the corresponding entry in the procedure linkage table,
-	         plus the desired section offset.
-	         If its value is zero then the dynamic linker has to resolve
-	         the symbol. We are unable to find any meaningful address
-	         for this symbol in the executable file, so we skip it.  */
-	      symaddr = sym->value;
-	      if (symaddr == 0)
-		continue;
-	      symaddr += offset;
-	      msym = record_minimal_symbol
-		((char *) sym->name, symaddr,
-		 mst_solib_trampoline, sym->section, objfile);
-#ifdef SOFUN_ADDRESS_MAYBE_MISSING
-	      if (msym != NULL)
-		msym->filename = filesymname;
-#endif
-	      continue;
-	    }
-
-	  /* If it is a nonstripped executable, do not enter dynamic
-	     symbols, as the dynamic symbol table is usually a subset
-	     of the main symbol table.  */
-	  if (dynamic && !stripped)
+	  /* Symbol is a reference to a function defined in
+	     a shared library.
+	     If its value is non zero then it is usually the address
+	     of the corresponding entry in the procedure linkage table,
+	     plus the desired section offset.
+	     If its value is zero then the dynamic linker has to resolve
+	     the symbol. We are unable to find any meaningful address
+	     for this symbol in the executable file, so we skip it.  */
+	  symaddr = sym->value;
+	  if (symaddr == 0)
 	    continue;
-	  if (sym->flags & BSF_FILE)
-	    {
-	      /* STT_FILE debugging symbol that helps stabs-in-elf debugging.
-	         Chain any old one onto the objfile; remember new sym.  */
-	      if (sectinfo != NULL)
-		{
-		  sectinfo->next = dbx->stab_section_info;
-		  dbx->stab_section_info = sectinfo;
-		  sectinfo = NULL;
-		}
-	      filesym = sym;
+	  symaddr += offset;
+	  msym = record_minimal_symbol
+	    ((char *) sym->name, symaddr,
+	     mst_solib_trampoline, sym->section, objfile);
 #ifdef SOFUN_ADDRESS_MAYBE_MISSING
-	      filesymname =
-		obsavestring ((char *) filesym->name, strlen (filesym->name),
-			      &objfile->objfile_obstack);
+	  if (msym != NULL)
+	    msym->filename = filesymname;
 #endif
-	    }
-	  else if (sym->flags & (BSF_GLOBAL | BSF_LOCAL | BSF_WEAK))
-	    {
-	      struct minimal_symbol *msym;
+	  continue;
+	}
 
-	      /* Select global/local/weak symbols.  Note that bfd puts abs
-	         symbols in their own section, so all symbols we are
-	         interested in will have a section. */
-	      /* Bfd symbols are section relative. */
-	      symaddr = sym->value + sym->section->vma;
-	      /* Relocate all non-absolute symbols by the section offset.  */
-	      if (sym->section != &bfd_abs_section)
+      /* If it is a nonstripped executable, do not enter dynamic
+	 symbols, as the dynamic symbol table is usually a subset
+	 of the main symbol table.  */
+      if (dynamic && !stripped)
+	continue;
+      if (sym->flags & BSF_FILE)
+	{
+	  /* STT_FILE debugging symbol that helps stabs-in-elf debugging.
+	     Chain any old one onto the objfile; remember new sym.  */
+	  if (sectinfo != NULL)
+	    {
+	      sectinfo->next = dbx->stab_section_info;
+	      dbx->stab_section_info = sectinfo;
+	      sectinfo = NULL;
+	    }
+	  filesym = sym;
+#ifdef SOFUN_ADDRESS_MAYBE_MISSING
+	  filesymname =
+	    obsavestring ((char *) filesym->name, strlen (filesym->name),
+			  &objfile->objfile_obstack);
+#endif
+	}
+      else if (sym->flags & BSF_SECTION_SYM)
+	continue;
+      else if (sym->flags & (BSF_GLOBAL | BSF_LOCAL | BSF_WEAK))
+	{
+	  struct minimal_symbol *msym;
+
+	  /* Select global/local/weak symbols.  Note that bfd puts abs
+	     symbols in their own section, so all symbols we are
+	     interested in will have a section. */
+	  /* Bfd symbols are section relative. */
+	  symaddr = sym->value + sym->section->vma;
+	  /* Relocate all non-absolute symbols by the section offset.  */
+	  if (sym->section != &bfd_abs_section)
+	    {
+	      symaddr += offset;
+	    }
+	  /* For non-absolute symbols, use the type of the section
+	     they are relative to, to intuit text/data.  Bfd provides
+	     no way of figuring this out for absolute symbols. */
+	  if (sym->section == &bfd_abs_section)
+	    {
+	      /* This is a hack to get the minimal symbol type
+		 right for Irix 5, which has absolute addresses
+		 with special section indices for dynamic symbols. */
+	      unsigned short shndx =
+		((elf_symbol_type *) sym)->internal_elf_sym.st_shndx;
+
+	      switch (shndx)
 		{
+		case SHN_MIPS_TEXT:
+		  ms_type = mst_text;
+		  break;
+		case SHN_MIPS_DATA:
+		  ms_type = mst_data;
+		  break;
+		case SHN_MIPS_ACOMMON:
+		  ms_type = mst_bss;
+		  break;
+		default:
+		  ms_type = mst_abs;
+		}
+
+	      /* If it is an Irix dynamic symbol, skip section name
+		 symbols, relocate all others by section offset. */
+	      if (ms_type != mst_abs)
+		{
+		  if (sym->name[0] == '.')
+		    continue;
 		  symaddr += offset;
 		}
-	      /* For non-absolute symbols, use the type of the section
-	         they are relative to, to intuit text/data.  Bfd provides
-	         no way of figuring this out for absolute symbols. */
-	      if (sym->section == &bfd_abs_section)
+	    }
+	  else if (sym->section->flags & SEC_CODE)
+	    {
+	      if (sym->flags & BSF_GLOBAL)
 		{
-		  /* This is a hack to get the minimal symbol type
-		     right for Irix 5, which has absolute addresses
-		     with special section indices for dynamic symbols. */
-		  unsigned short shndx =
-		  ((elf_symbol_type *) sym)->internal_elf_sym.st_shndx;
-
-		  switch (shndx)
+		  ms_type = mst_text;
+		}
+	      else if ((sym->name[0] == '.' && sym->name[1] == 'L')
+		       || ((sym->flags & BSF_LOCAL)
+			   && sym->name[0] == '$'
+			   && sym->name[1] == 'L'))
+		/* Looks like a compiler-generated label.  Skip
+		   it.  The assembler should be skipping these (to
+		   keep executables small), but apparently with
+		   gcc on the (deleted) delta m88k SVR4, it loses.
+		   So to have us check too should be harmless (but
+		   I encourage people to fix this in the assembler
+		   instead of adding checks here).  */
+		continue;
+	      else
+		{
+		  ms_type = mst_file_text;
+		}
+	    }
+	  else if (sym->section->flags & SEC_ALLOC)
+	    {
+	      if (sym->flags & (BSF_GLOBAL | BSF_WEAK))
+		{
+		  if (sym->section->flags & SEC_LOAD)
 		    {
-		    case SHN_MIPS_TEXT:
-		      ms_type = mst_text;
-		      break;
-		    case SHN_MIPS_DATA:
 		      ms_type = mst_data;
-		      break;
-		    case SHN_MIPS_ACOMMON:
+		    }
+		  else
+		    {
 		      ms_type = mst_bss;
-		      break;
-		    default:
-		      ms_type = mst_abs;
-		    }
-
-		  /* If it is an Irix dynamic symbol, skip section name
-		     symbols, relocate all others by section offset. */
-		  if (ms_type != mst_abs)
-		    {
-		      if (sym->name[0] == '.')
-			continue;
-		      symaddr += offset;
 		    }
 		}
-	      else if (sym->section->flags & SEC_CODE)
+	      else if (sym->flags & BSF_LOCAL)
 		{
-		  if (sym->flags & BSF_GLOBAL)
-		    {
-		      ms_type = mst_text;
-		    }
-		  else if ((sym->name[0] == '.' && sym->name[1] == 'L')
-			   || ((sym->flags & BSF_LOCAL)
-			       && sym->name[0] == '$'
-			       && sym->name[1] == 'L'))
-		    /* Looks like a compiler-generated label.  Skip
-		       it.  The assembler should be skipping these (to
-		       keep executables small), but apparently with
-		       gcc on the (deleted) delta m88k SVR4, it loses.
-		       So to have us check too should be harmless (but
-		       I encourage people to fix this in the assembler
-		       instead of adding checks here).  */
-		    continue;
+		  /* Named Local variable in a Data section.
+		     Check its name for stabs-in-elf.  */
+		  int special_local_sect;
+		  if (strcmp ("Bbss.bss", sym->name) == 0)
+		    special_local_sect = SECT_OFF_BSS (objfile);
+		  else if (strcmp ("Ddata.data", sym->name) == 0)
+		    special_local_sect = SECT_OFF_DATA (objfile);
+		  else if (strcmp ("Drodata.rodata", sym->name) == 0)
+		    special_local_sect = SECT_OFF_RODATA (objfile);
 		  else
+		    special_local_sect = -1;
+		  if (special_local_sect >= 0)
 		    {
-		      ms_type = mst_file_text;
-		    }
-		}
-	      else if (sym->section->flags & SEC_ALLOC)
-		{
-		  if (sym->flags & (BSF_GLOBAL | BSF_WEAK))
-		    {
-		      if (sym->section->flags & SEC_LOAD)
+		      /* Found a special local symbol.  Allocate a
+			 sectinfo, if needed, and fill it in.  */
+		      if (sectinfo == NULL)
 			{
-			  ms_type = mst_data;
-			}
-		      else
-			{
-			  ms_type = mst_bss;
-			}
-		    }
-		  else if (sym->flags & BSF_LOCAL)
-		    {
-		      /* Named Local variable in a Data section.
-		         Check its name for stabs-in-elf.  */
-		      int special_local_sect;
-		      if (strcmp ("Bbss.bss", sym->name) == 0)
-			special_local_sect = SECT_OFF_BSS (objfile);
-		      else if (strcmp ("Ddata.data", sym->name) == 0)
-			special_local_sect = SECT_OFF_DATA (objfile);
-		      else if (strcmp ("Drodata.rodata", sym->name) == 0)
-			special_local_sect = SECT_OFF_RODATA (objfile);
-		      else
-			special_local_sect = -1;
-		      if (special_local_sect >= 0)
-			{
-			  /* Found a special local symbol.  Allocate a
-			     sectinfo, if needed, and fill it in.  */
-			  if (sectinfo == NULL)
+			  int max_index;
+			  size_t size;
+
+			  max_index 
+			    = max (SECT_OFF_BSS (objfile),
+				   max (SECT_OFF_DATA (objfile),
+					SECT_OFF_RODATA (objfile)));
+
+			  /* max_index is the largest index we'll
+			     use into this array, so we must
+			     allocate max_index+1 elements for it.
+			     However, 'struct stab_section_info'
+			     already includes one element, so we
+			     need to allocate max_index aadditional
+			     elements.  */
+			  size = (sizeof (struct stab_section_info) 
+				  + (sizeof (CORE_ADDR)
+				     * max_index));
+			  sectinfo = (struct stab_section_info *)
+			    xmalloc (size);
+			  memset (sectinfo, 0, size);
+			  sectinfo->num_sections = max_index;
+			  if (filesym == NULL)
 			    {
-			      int max_index;
-			      size_t size;
-
-			      max_index 
-				= max (SECT_OFF_BSS (objfile),
-				       max (SECT_OFF_DATA (objfile),
-					    SECT_OFF_RODATA (objfile)));
-
-                              /* max_index is the largest index we'll
-                                 use into this array, so we must
-                                 allocate max_index+1 elements for it.
-                                 However, 'struct stab_section_info'
-                                 already includes one element, so we
-                                 need to allocate max_index aadditional
-                                 elements.  */
-			      size = (sizeof (struct stab_section_info) 
-				      + (sizeof (CORE_ADDR)
-					 * max_index));
-			      sectinfo = (struct stab_section_info *)
-				xmmalloc (objfile->md, size);
-			      memset (sectinfo, 0, size);
-			      sectinfo->num_sections = max_index;
-			      if (filesym == NULL)
-				{
-				  complaint (&symfile_complaints,
-					     "elf/stab section information %s without a preceding file symbol",
-					     sym->name);
-				}
-			      else
-				{
-				  sectinfo->filename =
-				    (char *) filesym->name;
-				}
+			      complaint (&symfile_complaints,
+					 _("elf/stab section information %s without a preceding file symbol"),
+					 sym->name);
+			    }
+			  else
+			    {
+			      sectinfo->filename =
+				(char *) filesym->name;
 			    }
-			  if (sectinfo->sections[special_local_sect] != 0)
-			    complaint (&symfile_complaints,
-				       "duplicated elf/stab section information for %s",
-				       sectinfo->filename);
-			  /* BFD symbols are section relative.  */
-			  symaddr = sym->value + sym->section->vma;
-			  /* Relocate non-absolute symbols by the
-                             section offset.  */
-			  if (sym->section != &bfd_abs_section)
-			    symaddr += offset;
-			  sectinfo->sections[special_local_sect] = symaddr;
-			  /* The special local symbols don't go in the
-			     minimal symbol table, so ignore this one.  */
-			  continue;
-			}
-		      /* Not a special stabs-in-elf symbol, do regular
-		         symbol processing.  */
-		      if (sym->section->flags & SEC_LOAD)
-			{
-			  ms_type = mst_file_data;
-			}
-		      else
-			{
-			  ms_type = mst_file_bss;
 			}
+		      if (sectinfo->sections[special_local_sect] != 0)
+			complaint (&symfile_complaints,
+				   _("duplicated elf/stab section information for %s"),
+				   sectinfo->filename);
+		      /* BFD symbols are section relative.  */
+		      symaddr = sym->value + sym->section->vma;
+		      /* Relocate non-absolute symbols by the
+			 section offset.  */
+		      if (sym->section != &bfd_abs_section)
+			symaddr += offset;
+		      sectinfo->sections[special_local_sect] = symaddr;
+		      /* The special local symbols don't go in the
+			 minimal symbol table, so ignore this one.  */
+		      continue;
+		    }
+		  /* Not a special stabs-in-elf symbol, do regular
+		     symbol processing.  */
+		  if (sym->section->flags & SEC_LOAD)
+		    {
+		      ms_type = mst_file_data;
 		    }
 		  else
 		    {
-		      ms_type = mst_unknown;
+		      ms_type = mst_file_bss;
 		    }
 		}
 	      else
 		{
-		  /* FIXME:  Solaris2 shared libraries include lots of
-		     odd "absolute" and "undefined" symbols, that play 
-		     hob with actions like finding what function the PC
-		     is in.  Ignore them if they aren't text, data, or bss.  */
-		  /* ms_type = mst_unknown; */
-		  continue;	/* Skip this symbol. */
+		  ms_type = mst_unknown;
 		}
-	      msym = record_minimal_symbol
-		((char *) sym->name, symaddr,
-		 ms_type, sym->section, objfile);
-	      if (msym)
-	      {
-		/* Pass symbol size field in via BFD.  FIXME!!!  */
-		unsigned long size = ((elf_symbol_type *) sym)->internal_elf_sym.st_size;
-		MSYMBOL_SIZE(msym) = size;
-	      }
-#ifdef SOFUN_ADDRESS_MAYBE_MISSING
-	      if (msym != NULL)
-		msym->filename = filesymname;
-#endif
-	      ELF_MAKE_MSYMBOL_SPECIAL (sym, msym);
 	    }
+	  else
+	    {
+	      /* FIXME:  Solaris2 shared libraries include lots of
+		 odd "absolute" and "undefined" symbols, that play 
+		 hob with actions like finding what function the PC
+		 is in.  Ignore them if they aren't text, data, or bss.  */
+	      /* ms_type = mst_unknown; */
+	      continue;	/* Skip this symbol. */
+	    }
+	  msym = record_minimal_symbol
+	    ((char *) sym->name, symaddr,
+	     ms_type, sym->section, objfile);
+	  if (msym)
+	    {
+	      /* Pass symbol size field in via BFD.  FIXME!!!  */
+	      unsigned long size = ((elf_symbol_type *) sym)->internal_elf_sym.st_size;
+	      MSYMBOL_SIZE(msym) = size;
+	    }
+#ifdef SOFUN_ADDRESS_MAYBE_MISSING
+	  if (msym != NULL)
+	    msym->filename = filesymname;
+#endif
+	  ELF_MAKE_MSYMBOL_SPECIAL (sym, msym);
 	}
-      do_cleanups (back_to);
     }
 }
 
@@ -491,6 +462,9 @@ elf_symfile_read (struct objfile *objfile, int mainline)
   struct elfinfo ei;
   struct cleanup *back_to;
   CORE_ADDR offset;
+  long symcount = 0, dynsymcount = 0, synthcount, storage_needed;
+  asymbol **symbol_table = NULL, **dyn_symbol_table = NULL;
+  asymbol *synthsyms;
 
   init_minimal_symbol_collection ();
   back_to = make_cleanup_discard_minimal_symbols ();
@@ -498,20 +472,74 @@ elf_symfile_read (struct objfile *objfile, int mainline)
   memset ((char *) &ei, 0, sizeof (ei));
 
   /* Allocate struct to keep track of the symfile */
-  objfile->sym_stab_info = (struct dbx_symfile_info *)
-    xmmalloc (objfile->md, sizeof (struct dbx_symfile_info));
-  memset ((char *) objfile->sym_stab_info, 0, sizeof (struct dbx_symfile_info));
+  objfile->deprecated_sym_stab_info = (struct dbx_symfile_info *)
+    xmalloc (sizeof (struct dbx_symfile_info));
+  memset ((char *) objfile->deprecated_sym_stab_info, 0, sizeof (struct dbx_symfile_info));
   make_cleanup (free_elfinfo, (void *) objfile);
 
   /* Process the normal ELF symbol table first.  This may write some 
-     chain of info into the dbx_symfile_info in objfile->sym_stab_info,
+     chain of info into the dbx_symfile_info in objfile->deprecated_sym_stab_info,
      which can later be used by elfstab_offset_sections.  */
 
-  elf_symtab_read (objfile, 0);
+  storage_needed = bfd_get_symtab_upper_bound (objfile->obfd);
+  if (storage_needed < 0)
+    error (_("Can't read symbols from %s: %s"), bfd_get_filename (objfile->obfd),
+	   bfd_errmsg (bfd_get_error ()));
+
+  if (storage_needed > 0)
+    {
+      symbol_table = (asymbol **) xmalloc (storage_needed);
+      make_cleanup (xfree, symbol_table);
+      symcount = bfd_canonicalize_symtab (objfile->obfd, symbol_table);
+
+      if (symcount < 0)
+	error (_("Can't read symbols from %s: %s"), bfd_get_filename (objfile->obfd),
+	       bfd_errmsg (bfd_get_error ()));
+
+      elf_symtab_read (objfile, 0, symcount, symbol_table);
+    }
 
   /* Add the dynamic symbols.  */
 
-  elf_symtab_read (objfile, 1);
+  storage_needed = bfd_get_dynamic_symtab_upper_bound (objfile->obfd);
+
+  if (storage_needed > 0)
+    {
+      dyn_symbol_table = (asymbol **) xmalloc (storage_needed);
+      make_cleanup (xfree, dyn_symbol_table);
+      dynsymcount = bfd_canonicalize_dynamic_symtab (objfile->obfd,
+						     dyn_symbol_table);
+
+      if (dynsymcount < 0)
+	error (_("Can't read symbols from %s: %s"), bfd_get_filename (objfile->obfd),
+	       bfd_errmsg (bfd_get_error ()));
+
+      elf_symtab_read (objfile, 1, dynsymcount, dyn_symbol_table);
+    }
+
+  /* Add synthetic symbols - for instance, names for any PLT entries.  */
+
+  synthcount = bfd_get_synthetic_symtab (abfd, symcount, symbol_table,
+					 dynsymcount, dyn_symbol_table,
+					 &synthsyms);
+  if (synthcount > 0)
+    {
+      asymbol **synth_symbol_table;
+      long i;
+
+      make_cleanup (xfree, synthsyms);
+      synth_symbol_table = xmalloc (sizeof (asymbol *) * synthcount);
+      for (i = 0; i < synthcount; i++)
+	{
+	  synth_symbol_table[i] = synthsyms + i;
+	  /* Synthetic symbols are not, strictly speaking, either local
+	     or global.  But we can treat them as global symbols, since
+	     they are effectively dynamic symbols.  */
+	  synth_symbol_table[i]->flags |= BSF_GLOBAL;
+	}
+      make_cleanup (xfree, synth_symbol_table);
+      elf_symtab_read (objfile, 0, synthcount, synth_symbol_table);
+    }
 
   /* Install any minimal symbols that have been collected as the current
      minimal symbols for this objfile.  The debug readers below this point
@@ -594,21 +622,22 @@ elf_symfile_read (struct objfile *objfile, int mainline)
   dwarf2_build_frame_info (objfile);
 }
 
-/* This cleans up the objfile's sym_stab_info pointer, and the chain of
-   stab_section_info's, that might be dangling from it.  */
+/* This cleans up the objfile's deprecated_sym_stab_info pointer, and
+   the chain of stab_section_info's, that might be dangling from
+   it.  */
 
 static void
 free_elfinfo (void *objp)
 {
   struct objfile *objfile = (struct objfile *) objp;
-  struct dbx_symfile_info *dbxinfo = objfile->sym_stab_info;
+  struct dbx_symfile_info *dbxinfo = objfile->deprecated_sym_stab_info;
   struct stab_section_info *ssi, *nssi;
 
   ssi = dbxinfo->stab_section_info;
   while (ssi)
     {
       nssi = ssi->next;
-      xmfree (objfile->md, ssi);
+      xfree (ssi);
       ssi = nssi;
     }
 
@@ -637,9 +666,9 @@ elf_new_init (struct objfile *ignore)
 static void
 elf_symfile_finish (struct objfile *objfile)
 {
-  if (objfile->sym_stab_info != NULL)
+  if (objfile->deprecated_sym_stab_info != NULL)
     {
-      xmfree (objfile->md, objfile->sym_stab_info);
+      xfree (objfile->deprecated_sym_stab_info);
     }
 }
 
@@ -673,7 +702,7 @@ void
 elfstab_offset_sections (struct objfile *objfile, struct partial_symtab *pst)
 {
   char *filename = pst->filename;
-  struct dbx_symfile_info *dbx = objfile->sym_stab_info;
+  struct dbx_symfile_info *dbx = objfile->deprecated_sym_stab_info;
   struct stab_section_info *maybe = dbx->stab_section_info;
   struct stab_section_info *questionable = 0;
   int i;
@@ -703,7 +732,7 @@ elfstab_offset_sections (struct objfile *objfile, struct partial_symtab *pst)
   if (maybe == 0 && questionable != 0)
     {
       complaint (&symfile_complaints,
-		 "elf/stab section information questionable for %s", filename);
+		 _("elf/stab section information questionable for %s"), filename);
       maybe = questionable;
     }
 
@@ -722,7 +751,7 @@ elfstab_offset_sections (struct objfile *objfile, struct partial_symtab *pst)
   /* We were unable to find any offsets for this file.  Complain.  */
   if (dbx->stab_section_info)	/* If there *is* any info, */
     complaint (&symfile_complaints,
-	       "elf/stab section information missing for %s", filename);
+	       _("elf/stab section information missing for %s"), filename);
 }
 
 /* Register that we are able to handle ELF object file formats.  */
