@@ -119,6 +119,7 @@ proc gdbtk_tcl_preloop { } {
     set current_args [gdb_get_inferior_args]
     set current_dir $gdb_current_directory
     Session::notice_file_change
+    set_baud
     if {[string length $current_args] > 0} {
       gdb_set_inferior_args $current_args
       gdb_cmd "cd $current_dir"
@@ -364,6 +365,9 @@ proc gdbtk_tcl_warning {message} {
 
   switch -regexp $message {
         "Unable to find dynamic linker breakpoint function.*" {return}
+	"Internal error.*" { gdbtk_tcl_fputs_error $message }
+        "incomplete CFI.*" { gdbtk_tcl_fputs_error $message }
+	"RTTI symbol not found for class.*" { gdbtk_tcl_fputs_error $message }
         default {show_warning $message}
        }
 }
@@ -434,7 +438,7 @@ proc echo {args} {
 # PROC: gdbtk_tcl_fputs_error - write an error message
 # ------------------------------------------------------------------
 proc gdbtk_tcl_fputs_error {message} {
-  if {[info exists gdbtk_state(console)] && $::gdbtk_state(console) != ""} {
+  if {[info exists ::gdbtk_state(console)] && $::gdbtk_state(console) != ""} {
     $::gdbtk_state(console) insert $message err_tag
     update
   }
@@ -444,7 +448,7 @@ proc gdbtk_tcl_fputs_error {message} {
 # PROC: gdbtk_tcl_fputs_log - write a log message
 # ------------------------------------------------------------------
 proc gdbtk_tcl_fputs_log {message} {
-  if {[info exists gdbtk_state(console)] && $::gdbtk_state(console) != ""} {
+  if {[info exists ::gdbtk_state(console)] && $::gdbtk_state(console) != ""} {
     $::gdbtk_state(console) insert $message log_tag
     update
   }
@@ -701,10 +705,10 @@ proc gdbtk_memory_changed {} {
 #         - file_changed_hook                                      #
 #            Called in file_command. The tcl hook is               #
 #            "gdbtk_tcl_file_changed"                              #
-#         - pre_add_symbol_hook                                    #
+#         - deprecated_pre_add_symbol_hook                         #
 #            Called in symbol_file_add before loading. The tcl     #
 #            hook is "gdbtk_tcl_pre_add_symbol"                    #
-#         - post_add_symbol_hook                                   #
+#         - deprecated_post_add_symbol_hook                        #
 #            Called in symbol_file_add when finished loading       #
 #            a symbol file. The tcl hook is                        #
 #            "gdbtk_tcl_post_add_symbol"                           #
@@ -1296,7 +1300,7 @@ proc run_executable { {auto_start 1} } {
     } else {
       SrcWin::point_to_main
     }
-
+    
     gdbtk_update
     gdbtk_idle
   } elseif {[pref get gdb/mode]} {
@@ -1492,15 +1496,16 @@ proc gdbtk_detached {} {
 # the debugger must be able to keep gui alive while target_wait is
 # blocking (so that the user can interrupt or detach from it).
 # 
-# The best solution for this is to capture gdb deep down where it
-# can block. For _any_ target board, this will be in either
-# serial or socket code. These places call ui_loop_hook to 
-# keep us alive. For native unix, we use an interval timer.
-# Simulators either call ui_loop_hook directly (older sims, at least)
-# or they call gdb's os_poll_quit callback, where we insert a call
-# to ui_loop_hook. Some targets (like v850ice and windows native)
-# require a call to ui_loop_hook directly in target_wait. See comments
-# before gdb_stop and x_event to find out more about how this is accomplished.
+# The best solution for this is to capture gdb deep down where it can
+# block. For _any_ target board, this will be in either serial or
+# socket code. These places call deprecated_ui_loop_hook to keep us
+# alive. For native unix, we use an interval timer.  Simulators either
+# call deprecated_ui_loop_hook directly (older sims, at least) or they
+# call gdb's os_poll_quit callback, where we insert a call to
+# deprecated_ui_loop_hook. Some targets (like v850ice and windows
+# native) require a call to deprecated_ui_loop_hook directly in
+# target_wait. See comments before gdb_stop and x_event to find out
+# more about how this is accomplished.
 #
 # The stop button's behavior:
 # Pressing the stop button should attempt to stop the target. If, after
